import { filter, find, is } from '../fn.js'
import { map, maybe, seq } from '../parser.js'
import { word } from './common.js'
import { useClause } from './use.js'
import { matchClause } from './match.js'
import { whereClause } from './where.js'
import { returnClause } from './return.js'
import { Filter, Match, Query, ReturnValues, SelectedGraph } from './types.js'

const use = find(is(SelectedGraph))
const match = filter(is(Match))
const where = filter(is(Filter))
const rv = find(is(ReturnValues))

const collect = clauses => {
  return new Query(
    use(clauses),
    match(clauses),
    where(clauses),
    rv(clauses)
  )
}

export const query = map(
  collect,
  seq(
    maybe(word(useClause)),
    word(matchClause),
    maybe(word(whereClause)),
    word(returnClause)
  )
)

