import { construct } from '../fn.js'
import { always, any, char, manyUntil, map, map1, noCaseString, seq, skip, string } from '../parser.js'
import { keyword, Keyword, Symbols, value, word, ws } from './common.js'
import { And, Equal, Filter, GreaterThan, GreaterThanEqual, LesserThan, LesserThanEqual, Literal, None, Not, Or, Xor } from './types.js'

const { Left, Right } = Symbols.Bracket.Round
const lparen = map1(construct(Literal), Left)
const rparen = map1(construct(Literal), Right)

// type and existence
export const nil = always(None, noCaseString('null'))
export const is = noCaseString('is')

// binary
export const gt = always(GreaterThan, char('>'))
export const gte = always(GreaterThanEqual, string('>='))
export const lt = always(LesserThan, char('<'))
export const lte = always(LesserThanEqual, string('<='))
export const eq = always(Equal, char('='))

export const and = always(And, noCaseString('and'))
export const or = always(Or, noCaseString('or'))
export const xor = always(Xor, noCaseString('xor'))

export const compare = any(gte, lte, gt, lt, eq)
export const logic = any(and, or, xor)

// unary
export const not = always(Not, noCaseString('not'))
export const unary = any(not)

export const expr = any(lparen, rparen, compare, logic, unary, value)
export const params = map(construct(Filter), manyUntil(keyword, word(expr)))

export const whereClause = seq(skip(Keyword.Where), ws, params)
